package com.gic.haoban.manage.web.controller.hm;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.api.dto.welcome.dto.WelcomeDetailDTO;
import com.gic.haoban.manage.api.service.hm.HmLinkApiService;
import com.gic.haoban.manage.api.service.hm.HmPageTemplateApiService;
import com.gic.haoban.manage.api.service.out.WelcomeApiService;
import com.gic.haoban.manage.web.controller.WebBaseController;

/**
 * 
* @ClassName: HmLinkController
* @Description: 活码链接
* @author xugh
* @date 2022年7月1日 下午2:15:39
*
 */
@RestController
@RequestMapping("/hm/link")
public class HmLinkController  {

    @Autowired
    private HmLinkApiService hmLinkApiService;
    @Autowired
    private HmPageTemplateApiService hmPageTemplateApiService;
    @Autowired
    private WelcomeApiService welcomeApiService;

    @RequestMapping("add")
    public RestResponse save(@RequestBody HmLinkDTO dto) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        dto.setCreatorId(loginUser.getClerkId());
        dto.setCreatorName(loginUser.getClerkName());
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        dto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        dto.setEnterpriseId(loginUser.getEnterpriseId());
        ServiceResponse<HmLinkDTO> resp = this.hmLinkApiService.save(dto);
        if (resp.isSuccess()) {
            String logContent = "新增引流链接【" + resp.getResult().getLinkCode() + "-" + dto.getName() + "】";
        }
        return RestResponse.successResult();
    }

    @RequestMapping("update")
    public RestResponse update(@RequestBody HmLinkDTO dto) {
        Long linkId = dto.getLinkId();
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        ServiceResponse<HmLinkDTO> resp = this.hmLinkApiService.getByLinkId(loginUser.getWxEnterpriseId(),
                loginUser.getEnterpriseId(), linkId);
        HmLinkDTO oldDTO = resp.getResult();
        dto.setModifierId(loginUser.getClerkId());
        dto.setModifierName(loginUser.getClerkName());
        this.hmLinkApiService.save(dto);
        String logContent = this.getLog(oldDTO, dto);
        return RestResponse.successResult();
    }

    // 编辑引流活码【链接ID-链接名称】：链接名称「 链接名称」变更「 链接名称」，链接描述「 链接描述」变更「 链接描述」，链接标签「
    // 标签名称-标签值名称」变更为「 标签名称-标签值名称」，分配规则「 层级x-子项名称」关闭，「 层级x-子项名称」开启，欢迎语「
    // 欢迎语名称」变更为「 欢迎语名称」，欢迎语变更为「 不发送欢迎语」，欢迎语变更为「 活码欢迎语-欢迎语名称」
    private String getLog(HmLinkDTO oldDTO, HmLinkDTO newDTO) {
        StringBuilder sb = new StringBuilder("编辑引流活码").append("【").append(oldDTO.getLinkCode()).append("-")
                .append(oldDTO.getName()).append("】");
        comp("链接名称", oldDTO.getName(), newDTO.getName(), sb);
        comp("链接描述", oldDTO.getRemark(), newDTO.getRemark(), sb);
        
        
        return null;
    }

    public void comp(String title, String oldValue, String newValue, StringBuilder sb) {
        if (!oldValue.equals(newValue)) {
            sb.append(title).append("「").append(oldValue).append("」变更「").append(newValue).append("」");
        }
    }

    @RequestMapping("page")
    public RestResponse page(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        qdto.setWxEnterpriseId(loginUser.getWxEnterpriseId());
        qdto.setEnterpriseId(loginUser.getEnterpriseId());
        ServiceResponse<Page<HmLinkDTO>> page = this.hmLinkApiService.listPage(qdto, basePageInfo);
        return RestResponse.successResult(page.getResult());
    }

    @RequestMapping("detail")
    public RestResponse detail(Long linkId) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        String enterpriseId = loginUser.getEnterpriseId();
        ServiceResponse<HmLinkDTO> linkResp = this.hmLinkApiService.getByLinkId(wxEnterpriseId, enterpriseId, linkId);
        if (!linkResp.isSuccess()) {
            return RestResponse.failure("1", "链接不存在");
        }
        HmLinkDTO link = linkResp.getResult();
        Map<String, Object> retMap = new HashMap<>();
        retMap.put("link", link);
        // 落地页
        Long pageId = link.getPageId();
        if (null != pageId) {
            // this.hmPageTemplateApiService.s
        }
        // 查询欢迎语
        String welcomeId = link.getWelcomeId();
        if (StringUtils.isNotBlank(welcomeId)) {
            ServiceResponse<WelcomeDetailDTO> resp = this.welcomeApiService.getWelcome(welcomeId, enterpriseId);
            if (resp.isSuccess() && null != resp.getResult()) {
                retMap.put("welcome", resp.getResult());
            }
        }
        return RestResponse.successResult(retMap);
    }

}
