package com.gic.haoban.manage.web.controller.hm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.clerk.api.dto.ClerkStoreListDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeListDTO;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.api.service.StaffClerkRelationApiService;
import com.gic.haoban.manage.api.service.hm.HmQrcodeApiService;
import com.gic.haoban.manage.web.vo.hm.ClerkListVO;
import com.gic.haoban.manage.web.vo.hm.ClerkStoreListVO;

@RestController
public class HmStoreController {

    private static final Logger logger = LoggerFactory.getLogger(HmStoreController.class);

    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;
    @Autowired
    private HmQrcodeApiService hmQrcodeApiService;

    @RequestMapping("store-clerk-list")
    @ResponseBody
    public RestResponse<Object> listStoreClerk(HttpServletRequest request, BasePageInfo basePageInfo) {
        WebLoginDTO loginUser = AuthWebRequestUtil.getLoginUser();
        // 分组id
        String storeGroupId = request.getParameter("storeGroupId");
        // 门店参数
        String storeSearchParams = request.getParameter("storeSearchParams");
        // 导购参数
        String clerkSearchParams = request.getParameter("clerkSearchParams");
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("enterpriseId", loginUser.getEnterpriseId());
        // 2线上
        params.put("status", 2);
        params.put("storeGroupId", storeGroupId);
        // 是否显示子分组(1显示,其余不显示)
        params.put("showChildren", 1);
        // 只获取导购和店长
        params.put("clerkTypeList", Arrays.asList(1, 0));
        if (StringUtils.isNotBlank(storeSearchParams)) {
            params.put("storeSearchParams", "%" + storeSearchParams + "%");
        }
        if (StringUtils.isNotBlank(clerkSearchParams)) {
            params.put("clerkSearchParams", "%" + clerkSearchParams + "%");
        }
        // 如果有权限控制,进行管辖门店过0滤
        List<String> authStoreIdList = this.staffApiService.getHaoBanStoreIdsRolesByClerkId(loginUser.getClerkId(),
                loginUser.getWxEnterpriseId());
        if (CollectionUtils.isEmpty(authStoreIdList)) {
            return RestResponse.failure("-1", "无授权门店");
        }
        if (!(authStoreIdList.size() == 1 && authStoreIdList.contains("-1"))) {
            params.put("storeIdList", authStoreIdList);
        }
        Page<ClerkStoreListDTO> page = new Page<>();
        page.setPageSize(basePageInfo.getPageSize());
        page.setCurrentPage(basePageInfo.getPageNum());
        page.setParams(params);
        logger.info("params:{}", JSON.toJSONString(params));
        page = this.clerkService.listStoreClerkByPage(page);
        Page<ClerkStoreListVO> retPage = new Page<>();
        retPage.setPageSize(basePageInfo.getPageSize());
        retPage.setCurrentPage(basePageInfo.getPageNum());
        retPage.setParams(params);
        retPage.setTotalCount(page.getTotalCount());
        retPage.setTotalPage(page.getTotalPage());
        List<ClerkStoreListVO> result = EntityUtil.changeEntityListByJSON(ClerkStoreListVO.class, page.getResult());
        // 好办状态
        this.haobanStatus(result);
        retPage.setResult(result);
        return RestResponse.successResult(retPage);
    }

    private void haobanStatus(List<ClerkStoreListVO> list) {
        List<String> clerkIdList = new ArrayList<>();
        if (CollectionUtils.isEmpty(list)) {
            for (ClerkStoreListVO store : list) {
                if (CollectionUtils.isNotEmpty(store.getClerkList())) {
                    List<String> tempList = store.getClerkList().stream().map(dto -> dto.getClerkId())
                            .collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(tempList)) {
                        clerkIdList.addAll(tempList);
                    }
                }
            }
        }
        // 未开启客户联系的
        List<String> status2List = new ArrayList<>();
        // 已有单人活码的
        List<String> status3List = new ArrayList<>();
        // 可以创建单人活码的
        List<String> status4List = new ArrayList<>();
        // 已关联企微号的,且开启客户联系
        List<String> statusList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(clerkIdList)) {
            List<StaffClerkRelationDTO> clerkRelationList = this.staffClerkRelationApiService
                    .listByClerkIds(clerkIdList);
            if (CollectionUtils.isNotEmpty(clerkRelationList)) {
                status2List.addAll(clerkRelationList.stream().filter(dto -> dto.getStatusFlag() == 1)
                        .map(dto -> dto.getClerkId()).collect(Collectors.toList()));
                statusList.addAll(clerkRelationList.stream().filter(dto -> dto.getStatusFlag() == 2)
                        .map(dto -> dto.getClerkId()).collect(Collectors.toList()));
            }
        }
        if (CollectionUtils.isNotEmpty(statusList)) {
            List<HmQrcodeListDTO> hmList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(hmList)) {
                status2List.addAll(hmList.stream().filter(dto -> dto.getStatusFlag() == 3).map(dto -> dto.getClerkId())
                        .collect(Collectors.toList()));
                status3List.addAll(hmList.stream().filter(dto -> dto.getStatusFlag() == 1).map(dto -> dto.getClerkId())
                        .collect(Collectors.toList()));
            }
            status4List
                    .addAll(statusList.stream().filter(id -> (!status2List.contains(id) && !status3List.contains(id)))
                            .collect(Collectors.toList()));
        }

        list.stream().forEach(item -> {
            for (ClerkListVO clerk : item.getClerkList()) {
                if (status2List.contains(clerk.getClerkId())) {
                    clerk.setHmStatus(2);
                }
                if (status3List.contains(clerk.getClerkId())) {
                    clerk.setHmStatus(3);
                }
                if (status4List.contains(clerk.getClerkId())) {
                    clerk.setHmStatus(4);
                }
            }
        });
    }
}
